/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import javax.swing.*;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.gui.SimpleMenuBar;
import org.greenstone.gatherer.gui.ModalDialog;
import org.greenstone.gatherer.util.Utility;

/** Displays a dynamic prompt to allow the user to choose how metadata is to be added, updated or removed from target FileNodes. The prompt changes depending on the action requested, the file nodes encountered and the number of file nodes in the selection. */
public class MetaEditPrompt
    extends ModalDialog 
    implements ActionListener {

    private int value;
    private JButton accumulate;
    private JButton accumulate_all;
    private JButton cancel;
    private JButton skip;
    private JButton overwrite;
    private JButton overwrite_all;
    private JButton remove;
    private JButton remove_all;
    private JButton update;
    private JButton update_all;

    static private Dimension LABEL_SIZE = new Dimension(100, 25);
    static private Dimension SIZE = new Dimension(712, 250);
    // Generic prompt values.
    static public int CONFIRM         =  0;
    static public int CANCEL          =  1;
    static public int SKIP            =  2;
    // Values for the different add and update action prompts.
    static public int ACCUMULATE      =  3;
    static public int ACCUMULATE_ALL  =  4;
    static public int OVERWRITE       =  5;
    static public int OVERWRITE_ALL   =  6;
    static public int UPDATE_ONCE     =  7; // Caused by SARM
    // Values for the different remove action prompts.
    static public int REMOVE          =  8;
    static public int REMOVE_ALL      =  9;
    // Prompt Types
    static public String ADD_PROMPT    = "MetaEditPrompt.Add_Prompt";
    static public String REMOVE_PROMPT = "MetaEditPrompt.Remove_Prompt";
    static public String UPDATE_PROMPT = "MetaEditPrompt.Overwrite_Prompt";

    public MetaEditPrompt(String type, boolean multiple_selection, File file, String element, String current_value, String new_value) {
	super(Gatherer.g_man, true); // Needed for modal response!

	// Setup
	this.setSize(SIZE);
	Dictionary.setText(this, "MetaEditPrompt.Title");

	if (type.equals(ADD_PROMPT)) {
	    this.setJMenuBar(new SimpleMenuBar("appendingmetadata"));
	}
	else if (type.equals(UPDATE_PROMPT)) {
	    this.setJMenuBar(new SimpleMenuBar("updatingmetadata"));
	}
	else if (type.equals(REMOVE_PROMPT)) {
	    this.setJMenuBar(new SimpleMenuBar("removingmetadata"));
	}

	// Creation
	JPanel content_pane = (JPanel)this.getContentPane();
	JLabel title_label = new JLabel();
	Dictionary.setText(title_label, type);

	JPanel details_pane = new JPanel();
	JPanel filename_panel = new JPanel();
	JLabel filename_label = new JLabel();
	filename_label.setPreferredSize(LABEL_SIZE);
	Dictionary.setText(filename_label, "MetaEditPrompt.File");
	JTextField filename_field = new JTextField(Utility.trimCenter(file.getAbsolutePath(), 120));
	filename_field.setEditable(false);
	filename_field.setFont(Gatherer.config.getFont("general.tooltip_font", false));
	filename_field.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));
	JPanel element_panel = new JPanel();
	JLabel element_label = new JLabel();
	element_label.setPreferredSize(LABEL_SIZE);
	Dictionary.setText(element_label, "MetaEditPrompt.Element");
	JTextField element_field = new JTextField(element);	
	element_field.setEditable(false);
	element_field.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));
	JPanel current_value_panel = new JPanel();
	JLabel current_value_label = new JLabel();
	current_value_label.setPreferredSize(LABEL_SIZE);
	Dictionary.setText(current_value_label, "MetaEditPrompt.Current_Value");
	JTextField current_value_field = new JTextField(current_value);
	current_value_field.setEditable(false);
	current_value_field.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));
	JPanel new_value_panel = new JPanel();
	JLabel new_value_label = new JLabel();
	new_value_label.setPreferredSize(LABEL_SIZE);
	Dictionary.setText(new_value_label, "MetaEditPrompt.New_Value");
	JTextField new_value_field = new JTextField(new_value); // , (type == ADD_PROMPT || type == UPDATE_PROMPT));
	new_value_field.setEditable(false);
	new_value_field.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));
	JPanel buttons_pane = new JPanel();

	accumulate = new JButton();
	accumulate.setEnabled(type == ADD_PROMPT);
	accumulate.setMnemonic(KeyEvent.VK_A);
	Dictionary.setBoth(accumulate, "MetaEdit.Accumulate", "MetaEdit.Accumulate_Tooltip");

	accumulate_all = new JButton();
	accumulate_all.setEnabled(type == ADD_PROMPT && multiple_selection);
	accumulate_all.setMnemonic(KeyEvent.VK_L);
	Dictionary.setBoth(accumulate_all, "MetaEditPrompt.Accumulate_All", "MetaEditPrompt.Accumulate_All_Tooltip");

	overwrite = new JButton();
	overwrite.setEnabled(type == UPDATE_PROMPT);
	overwrite.setMnemonic(KeyEvent.VK_R);
	Dictionary.setBoth(overwrite, "MetaEdit.Overwrite", "MetaEdit.Overwrite_Tooltip");

	overwrite_all = new JButton();
	overwrite_all.setEnabled(type == UPDATE_PROMPT && multiple_selection);
	overwrite_all.setMnemonic(KeyEvent.VK_P);
	Dictionary.setBoth(overwrite_all, "MetaEditPrompt.Overwrite_All", "MetaEditPrompt.Overwrite_All_Tooltip");

	remove = new JButton();
	remove.setEnabled(type == REMOVE_PROMPT);
	remove.setMnemonic(KeyEvent.VK_R);
	Dictionary.setBoth(remove, "MetaEdit.Remove", "MetaEdit.Remove_Tooltip");

	remove_all = new JButton();
	remove_all.setEnabled(type == REMOVE_PROMPT && multiple_selection);
	remove_all.setMnemonic(KeyEvent.VK_A);
	Dictionary.setBoth(remove_all, "MetaEditPrompt.Remove_All", "MetaEditPrompt.Remove_All_Tooltip");

	skip = new JButton();
	skip.setEnabled(multiple_selection);
	skip.setMnemonic(KeyEvent.VK_S);
	Dictionary.setBoth(skip, "MetaEditPrompt.Skip", "MetaEditPrompt.Skip_Tooltip");

	cancel = new JButton();
	cancel.setMnemonic(KeyEvent.VK_C);
	Dictionary.setBoth(cancel, "General.Cancel", "General.Cancel_Tooltip");

	// Connection
	accumulate.addActionListener(this);
	accumulate_all.addActionListener(this);
	cancel.addActionListener(this);
	overwrite.addActionListener(this);
	overwrite_all.addActionListener(this);
	remove.addActionListener(this);
	remove_all.addActionListener(this);
	skip.addActionListener(this);

	// Layout
	title_label.setBorder(BorderFactory.createEmptyBorder(2,2,2,2));
		  
	filename_panel.setLayout(new BorderLayout());
	filename_panel.add(filename_label, BorderLayout.WEST);
	filename_panel.add(filename_field, BorderLayout.CENTER);

	element_panel.setLayout(new BorderLayout());
	element_panel.add(element_label, BorderLayout.WEST);
	element_panel.add(element_field, BorderLayout.CENTER);

	current_value_panel.setLayout(new BorderLayout());
	current_value_panel.add(current_value_label, BorderLayout.WEST);
	current_value_panel.add(current_value_field, BorderLayout.CENTER);

	new_value_panel.setLayout(new BorderLayout());
	new_value_panel.add(new_value_label, BorderLayout.WEST);
	new_value_panel.add(new_value_field, BorderLayout.CENTER);

	details_pane.setBorder(BorderFactory.createEmptyBorder(2,2,2,2));
	details_pane.setLayout(new GridLayout(4,1,0,4));
	details_pane.add(filename_panel);
	details_pane.add(element_panel);
	details_pane.add(current_value_panel);
	details_pane.add(new_value_panel);

	buttons_pane.setBorder(BorderFactory.createEmptyBorder(2,2,2,2));
	buttons_pane.setLayout(new GridLayout(1,4,0,0));

	if (type.equals(ADD_PROMPT)) {
	    buttons_pane.add(accumulate);
	    buttons_pane.add(accumulate_all);
	}
	else if (type.equals(UPDATE_PROMPT)) {
	    buttons_pane.add(overwrite);
	    buttons_pane.add(overwrite_all);
	}
	else if (type.equals(REMOVE_PROMPT)) {
	    buttons_pane.add(remove);
	    buttons_pane.add(remove_all);
	}
  	buttons_pane.add(skip);
  	buttons_pane.add(cancel);

	content_pane.setBorder(BorderFactory.createEmptyBorder(3,3,3,3));
	content_pane.setLayout(new BorderLayout());
	content_pane.add(title_label, BorderLayout.NORTH);
	content_pane.add(details_pane, BorderLayout.CENTER);
	content_pane.add(buttons_pane, BorderLayout.SOUTH);

	// Position
	Rectangle frame_bounds = Gatherer.g_man.getBounds();
	this.setLocation(frame_bounds.x + ((frame_bounds.width - SIZE.width) / 2), frame_bounds.y + ((frame_bounds.height - SIZE.height) / 2));
    }

    public void actionPerformed(ActionEvent event) {
	Object esrc = event.getSource();
	if(esrc == accumulate) {
	    value = ACCUMULATE;
	}
	else if(esrc == accumulate_all) {
	    value = ACCUMULATE_ALL;
	}
	else if(esrc == cancel) {
	    value = CANCEL;
	}
	else if(esrc == skip) {
	    value = SKIP;
	}		  
	else if(esrc == overwrite) {
	    value = OVERWRITE;
	}
	else if(esrc == overwrite_all) {
	    value = OVERWRITE_ALL;
	}
	else if(esrc == remove) {
	    value = REMOVE;
	}
	else if(esrc == remove_all) {
	    value = REMOVE_ALL;
	}
	this.dispose();
    }

    public int display() {
	setVisible(true);
	return value;
    }
}
